# Generated by roxygen2: do not edit by hand

S3method("[",caretList)
S3method(as.caretList,default)
S3method(as.caretList,list)
S3method(autoplot,caretStack)
S3method(c,caretList)
S3method(c,train)
S3method(dotplot,caretStack)
S3method(extractCaretTarget,default)
S3method(extractCaretTarget,formula)
S3method(extractMetric,caretList)
S3method(extractMetric,caretStack)
S3method(extractMetric,train)
S3method(plot,caretList)
S3method(plot,caretStack)
S3method(predict,caretList)
S3method(predict,caretStack)
S3method(predict,greedyMSE)
S3method(print,caretStack)
S3method(print,greedyMSE)
S3method(print,summary.caretList)
S3method(print,summary.caretStack)
S3method(summary,caretList)
S3method(summary,caretStack)
S3method(varImp,caretStack)
S3method(varImp,greedyMSE)
export(as.caretList)
export(caretEnsemble)
export(caretList)
export(caretModelSpec)
export(caretStack)
export(defaultControl)
export(defaultMetric)
export(extractMetric)
export(greedyMSE)
export(greedyMSE_caret)
export(permutationImportance)
export(plot_variable_importance)
export(tuneCheck)
export(wtd.sd)
importFrom(caret,varImp)
importFrom(data.table,":=")
importFrom(data.table,.SD)
importFrom(ggplot2,autoplot)
importFrom(lattice,dotplot)
importFrom(patchwork,plot_layout)
importFrom(rlang,.data)
